package javax.xhttps;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.net.InetSocketAddress;
import java.security.KeyStore;

import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLEngine;
import javax.net.ssl.SSLParameters;
import javax.net.ssl.TrustManagerFactory;

import com.sun.net.httpserver.HttpExchange;
import com.sun.net.httpserver.HttpHandler;
import com.sun.net.httpserver.HttpsConfigurator;
import com.sun.net.httpserver.HttpsParameters;
import com.sun.net.httpserver.HttpsServer;
/**
 * xhttps : a tutorial project/ technical note about Java HTTP server.
 * 1. Zhikai Wang/www.heteroclinic.net does not claim any right upon this file and the whole integrated package (zip or tar).
 * 2. The liability of Zhikai Wang/www.heteroclinic.net to the maximum is to remove/modify matters in dispute.
 * 3. All third-party rights and licenses are automatically cascaded.
 * Based on 1,2 and 3, you can do anything with this file and the whole integrated package (zip or tar).
 * Nov 30, 2013 PST    
 * Please refer to the following links about this file as the major references. 
 * http://docs.oracle.com/javase/7/docs/jre/api/net/httpserver/spec/com/sun/net/httpserver/package-summary.html -- retrieved on Nov 25, 2013
 * http://stackoverflow.com/questions/2308479/simple-java-https-server  -- retrieved on Nov 30, 2013 PST
 * We comment: the above materials are Maven-able to resolve the proper dependencies under fair use condition for science and technology promotion purpose.
 * Open a browser and use this link to test result: https://127.0.0.1:8000/test 
 * At the bottom of this file documented how to create keystore use Java keytool.
 */
public class SSLHttpsServer {
	   public static void main(String[] args) throws Exception {
		   try
	        {
	            InetSocketAddress address = new InetSocketAddress(8000);
	            HttpsServer httpsServer = HttpsServer.create ( address, 0 );
	            SSLContext sslContext = SSLContext.getInstance ( "TLS" );

	            char[] password = "xhttps".toCharArray ();
	            KeyStore ks = KeyStore.getInstance ( "JKS" );
	           // FileInputStream fis = new FileInputStream ( "C:\\eclipseworkspace\\xhttps\\xhttps.jks" );
	            FileInputStream fis = new FileInputStream ( ClassLoader.getSystemResource("xhttps.jks").getFile() );
	            ks.load ( fis, password );

	            KeyManagerFactory kmf = KeyManagerFactory.getInstance ( "SunX509" );
	            kmf.init ( ks, password );

	            TrustManagerFactory tmf = TrustManagerFactory.getInstance ( "SunX509" );
	            tmf.init ( ks );

	            sslContext.init ( kmf.getKeyManagers (), tmf.getTrustManagers (), null );
	            httpsServer.setHttpsConfigurator ( new HttpsConfigurator( sslContext )
	            {
	                public void configure ( HttpsParameters params )
	                {
	                    try
	                    {
	                        SSLContext c = SSLContext.getDefault ();
	                        SSLEngine engine = c.createSSLEngine ();
	                        params.setNeedClientAuth ( false );
	                        params.setCipherSuites ( engine.getEnabledCipherSuites () );
	                        params.setProtocols ( engine.getEnabledProtocols () );
	                        SSLParameters defaultSSLParameters = c.getDefaultSSLParameters ();
	                        engine.setSSLParameters(defaultSSLParameters);
	                    }
	                    catch ( Exception ex )
	                    {
	                    	ex.printStackTrace();
	                    }
	                }
	            } );
	            httpsServer.createContext("/test", new MyHandler());
	            httpsServer.setExecutor(null); 
	            httpsServer.start();
	        }
	        catch ( Exception exception )
	        {
	        	exception.printStackTrace();
	        }
		   
	    }

	    static class MyHandler implements HttpHandler {
	        public void handle(HttpExchange t) throws IOException {
	            String response = "This is the response";
	            t.sendResponseHeaders(200, response.length());
	            OutputStream os = t.getResponseBody();
	            os.write(response.getBytes());
	            os.close();
	        }
	    }
}

// how to create keystore use Java keytool
//C:\eclipseworkspace\xhttps>keytool -genkey -alias xhttps -keystore xhttps.jks
//Enter keystore password: (xhttps)
//Re-enter new password:
//What is your first and last name?
//[Unknown]:  TEST
//What is the name of your organizational unit?
//[Unknown]:  heteroclinic.net
//What is the name of your organization?
//[Unknown]:  heteroclinic.net
//What is the name of your City or Locality?
//[Unknown]:  Montreal
//What is the name of your State or Province?
//[Unknown]:  Quebec
//What is the two-letter country code for this unit?
//[Unknown]:  CA
//Is CN=TEST, OU=heteroclinic.net, O=heteroclinic.net, L=Montreal, ST=Quebec, C=CA correct?
//[no]:  yes
//
//Enter key password for <xhttps>
//      (RETURN if same as keystore password):
//
//C:\eclipseworkspace\xhttps>dir
//Volume in drive C has no label.
//Volume Serial Number is 4C98-3338
//
//Directory of C:\eclipseworkspace\xhttps
//
//11/20/2013  11:10 PM    <DIR>          .
//11/20/2013  11:10 PM    <DIR>          ..
//11/12/2013  10:40 PM               705 .classpath
//11/11/2013  09:58 PM               558 .project
//11/12/2013  10:40 PM    <DIR>          .settings
//11/11/2013  10:04 PM             3,252 pom.xml
//11/11/2013  09:58 PM    <DIR>          src
//11/11/2013  10:08 PM    <DIR>          target
//11/20/2013  11:10 PM             1,317 xhttps.jks
//             4 File(s)          5,832 bytes
//             5 Dir(s)  244,771,729,408 bytes free
//
//C:\eclipseworkspace\xhttps>