package javax.xhttps;
import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicLong;
/**
 * xhttps : a tutorial project/ technical note about Java HTTP server.
 * 1. Zhikai Wang/www.heteroclinic.net does not claim any right upon this file and the whole integrated package (zip or tar).
 * 2. The liability of Zhikai Wang/www.heteroclinic.net to the maximum is to remove/modify matters in dispute.
 * 3. All third-party rights and licenses are automatically cascaded.
 * Based on 1,2 and 3, you can do anything with this file and the whole integrated package (zip or tar).
 * Nov 30, 2013 PST
 * This file demonstrate how to shut down a service program in protocol not a yank.
 * - for a process with console, input a special stop-code string
 * - ctrl-c or "service xhttps start/stop/restart", system will pass the signal    
 */
class Task implements Runnable {
	static protected AtomicLong count = new AtomicLong(0l);
	protected long iD = -1l;
	protected volatile boolean requestedStop = false;
	protected volatile boolean otherStop = false;

	public boolean isRequestedStop() {
		return requestedStop;
	}

	public void setRequestedStop(boolean requestedStop) {
		this.requestedStop = requestedStop;
	}

	public Task() {
		this.iD = count.incrementAndGet();
	}

	@Override
	public void run() {
		System.out.println("Task " + this.iD + " started.");
		try {
			while (!Thread.interrupted() && !requestedStop && !otherStop) {
				try {
					TimeUnit.MILLISECONDS.sleep(200);
				} catch (InterruptedException e) {
					// show or not?
					throw e;
				} catch (Exception e) {
					// show or not?
					//throw e;
					otherStop = true;
				}
			}
		} catch (Exception e) {
			// eat it
		} finally {
			System.out.println("Task " + this.iD + " ended.");
		}
	}

}

public class TwowaysofShutdown {
	static List<Future<?>> fl = new ArrayList<Future<?>>();
	static ExecutorService exec = Executors.newCachedThreadPool();
	
	static class HybridHttpServerShutdownHook extends Thread {// must be static

	
		public HybridHttpServerShutdownHook() {
		}

		@Override
		public void run() {
			try {
				Thread.currentThread().setName(
						"SHUTDOWNHOOK: TID-" + Thread.currentThread().getId());
				exec.shutdown();
				for (Future<?> f: fl)
					f.cancel(true);

			} catch (Exception e) {
				e.printStackTrace();
			} finally {
				System.out.println(this.getName() + " ended.");
			}
		}
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {

		fl.add(exec.submit(new Task()));
		fl.add(exec.submit(new Task()));
		
		
	    Runtime.getRuntime().addShutdownHook(
	                new HybridHttpServerShutdownHook());
		
		// The two ways to stop a server.
		// why use console, Eclipse shields ctrl-c
		// why shutdown hook, as a system service run without console, you can "service xhttps start/stop/restart"
		System.out.println("Type stop or use ctrl-c to terminate the service. ctrl-c not work in Eclipse");
		String condition = "";
		while (!condition.equalsIgnoreCase("stop")) {
			try {
				// create the Scanner
				Scanner scanner = new Scanner(System.in);

				// read input
				condition = scanner.nextLine();
			} catch (Throwable t) {
				t.printStackTrace();
				condition = "stop";
			}
		}

		// TODO call clean up
		exec.shutdown();
		for (Future<?> f: fl)
			f.cancel(true);
		System.out.println("If shutdown hook is called this line won't be called.");


	}

}
